;-----------------------------------------------------------------------;
; BINCLOCK.ASM          Hodiny zobrazujici v binarni soustave           ;
;-----------------------------------------------------------------------;

        LIST        P=16F628A, R=DEC    ; typ procesoru a defaultni cislicova soustava
        #include    "P16F628A.INC"      ; definice nazvu registru (jsou v tomto souboru)
        ERRORLEVEL  -224                ; vypnuti upozornovani na chyby v trisb
        ERRORLEVEL  -207                ; nezajem o spravne umisteni od okraje...

;-----------------------------------------------------------------------;
;         Nastaveni config. slov, konstant, promennych, direktiv        ;
;-----------------------------------------------------------------------;
#include    Head.inc

;-----------------------------------------------------------------------;
;                             Udani lokaci                              ;
;-----------------------------------------------------------------------;
            org     0           ; zacatek na 0
            goto    Main        ; skok na hlavni proceduru

            org     4
            goto    Interrupt   ; procedura obsluhy preruseni

;-----------------------------------------------------------------------;
;    Maximalni hodnoty + 1 zadane jednotky casu, uklada na registr W    ;
;-----------------------------------------------------------------------;
sethi:
            addwf   PCL, f                  ; ziska adresu na spravny registr
            dt      H'3C', H'3C', H'18'     ; tabulka maximalnich hodnot+1

SettingHi:
            addwf   PCL, f
            dt      H'08', H'3C', H'3C', H'18'
;-----------------------------------------------------------------------;
;           Vrati pin, ktery je treba zapnout pro rozsviceni LED        ;
;-----------------------------------------------------------------------;
Katoda:
            addwf   PCL, f
            dt      KAT_WEEK, KAT_SEC, KAT_MIN, KAT_HR, KAT_STATE

;-----------------------------------------------------------------------;
;                          Vraci pismena abecedy                        ;
;-----------------------------------------------------------------------;
Katoda_Char:                                ; odesila se v preskocnem formatu 1324
            addwf   PCL, f
            dt      KAT_WEEK, KAT_MIN, KAT_SEC, KAT_HR
Char_3:
            addwf   PCL, f
            dt      H'07', H'01', H'03', H'07'
Char_.:
            addwf   PCL, f
            dt      H'01', H'00', H'00', H'00'
Char_K:
            addwf   PCL, f
            dt      H'05', H'05', H'06', H'05'
Char_P:
            addwf   PCL, f
            dt      H'04', H'05', H'06', H'06'

;-----------------------------------------------------------------------;
;           Procedura pro prevod signalu na posuvny registr 4094        ;
;            Odesle registr W na 4094 a vypne jeho zobrazeni            ;
;-----------------------------------------------------------------------;
SendZnak:
            movwf   pom         ; znak ktery se bude odesilat predej do pom
            movlw   D'6'        ; cyklus odesila jen 6 bitu, vice neni potreba
            movwf   cykl
            rlf     pom, f
            rlf     pom, f      ; rotace na prvni potrebny bit
SendZnakLoop:
            bcf     OUT_D       ; vynulovani vstupu dat na 4094
            rlf     pom, f      ; vybrani bitu a jeho zapis na bit C registru STATUS
            btfsc   STATUS, C
            bsf     OUT_D       ; odesilat se bude 1
            bcf     OUT_CLK     ; odesilat se bude 0, nastav hranu clocku na 0
            bsf     OUT_CLK     ; hrana clocku na 1 => bit odeslan na 4094
            decfsz  cykl, f     ; test konce cyklu
            goto    SendZnakLoop; odesli dalsi bit

            clrf    PORTA       ; vypni zobrazeni displeje
            bcf     OUT_STB     ; Tvorba signalu
            bsf     OUT_STB     ; pro zapis dat
            bcf     OUT_STB     ; na piny 4094.
            return              ; cely znak jiz byl odeslan

;-----------------------------------------------------------------------;
;           Procedura cekani (mezi zobrazovanim hodnot casu)            ;
;-----------------------------------------------------------------------;
msec250:                        ; ziskani zpozdeni
            movlw   D'255'
nmsec:                          ; ceka X(X<256) milisekund, X je nastaveno v registru W
            nop                 ; instrukce nop trva 0,122 milisekund
            nop
            nop                 ; kazdy cyklus ma trvani 8 * 0,122 = 0,976 milisekund
            nop
            addlw   H'FF'       ; odcitani 1 od W
            btfss   STATUS, Z   ; je W=0 (Z=1)?
            goto    nmsec       ; ne, 2 * 0,122 milisekund
            return

nloop:
            addlw   H'FF'
            btfss   STATUS, Z
            goto    nloop
            return
;-----------------------------------------------------------------------;
;                           Doba jedne vteriny                          ;
;-----------------------------------------------------------------------;
onesecond:
            call    msec250
            call    msec250
            call    msec250
            call    msec250
            return

;-----------------------------------------------------------------------;
;                   Zobrazi aktualni cas na display                     ;
;-----------------------------------------------------------------------;
        ; pro 50Hz na kazdem registru musi disp_loop trvat 32 instrukcnich cyklu (=4 ns)
DispTime:
            movlw   week
            addwf   por_reg, W  ; prictenim adresy week k poradi ziska spravny odkaz
            movwf   FSR         ; Nastavi do FSR zpracovavany registr
            movf    INDF, W
            call    SendZnak     ; (preposle hodnotu registru na 4094)

            movf    por_reg, W  ; a pripravi jeho katodu
            call    Katoda
            movwf   PORTA       ; na spusteni.

            incf    por_reg, f

            movlw   D'5'        ; existuje 5 zobrazovanych registru
            subwf   por_reg, W
            btfsc   STATUS, Z   ; byly jiz vykresleny vsechny registry?
            call    DispTest    ; ano - vykonej par kontrol
            goto    DispTime    ; ne - vykresli dalsi registr
            return              ;  a potom se vrat
DispTest:   ; vsechny registry zobrazeny, osefuj ostatni
            clrf    por_reg     ; ano - vykresli znovu 1. registr

            btfsc   STATEB      ; kontrola stisku tl STATE                      ; reverze
            call    SetState
            btfsc   UNDEFB      ; kontrola dvojkliku - nastaveni casu
            call    CheckDouble
            movlw   R_1         ; stav pro stale zobrazeni
            subwf   state, W
            btfss   STATUS, Z
            decfsz  pom2
            return
            clrf    PORTA
            goto    MainLoop


;-----------------------------------------------------------------------;
;                   Zobrazi verzi a vyvojare firmware                   ;
;-----------------------------------------------------------------------;
DispVer:
            clrf    por_reg

DispVer1:
            movf    por_reg, W
            call    Char_3
            addwf   por_reg, W  ; prictenim adresy week k poradi ziska spravny odkaz
            movwf   FSR         ; Nastavi do FSR zpracovavany registr
            movf    INDF, W
            call    SendZnak     ; (preposle hodnotu registru na 4094)

            movf    por_reg, W  ; a pripravi jeho katodu
            call    Katoda
            movwf   PORTA       ; na spusteni.

            incf    por_reg, f

            movlw   D'5'        ; existuje 5 zobrazovanych registru
            subwf   por_reg, W
            btfsc   STATUS, Z   ; byly jiz vykresleny vsechny registry?
            call    DispTest    ; ano - vykonej par kontrol
            goto    DispTime    ; ne - vykresli dalsi registr
            return              ;  a potom se vrat


;-----------------------------------------------------------------------;
;                       Ceka na pusteni tlacitek                        ;
;-----------------------------------------------------------------------;
releaseSTATE:                   ; cekani na pusteni tlacitka STATE
            movlw   STATEB_MASK
            movwf   button
            goto    wait
releaseSELECT:                  ; cekani na pusteni tlacitka SELECT
            movlw   SELECTB_MASK
            movwf   button
            goto    wait
releaseSET:                     ; cekani na pusteni tlacitka SET
            movlw   SETB_MASK
            movwf   button
            goto    wait
releaseUNDEF:                    ; cekani na pusteni tlacitka UNDEF
            movlw   UNDEFB_MASK
            movwf   button
            goto    wait
wait:
            movf    button, W   ; presun masku na registr W
            andwf   PORTB, W
            btfss   STATUS, Z   ; preskoc pokud Z==0 tzn. tlacitko je pustene   ; reverze
            goto    wait
            movlw   D'10'
            call    nmsec       ; pockej 10 milisekund kvuli zakmitu
            movf    button, W   ; znovu zkontroluj pusteni
            andwf   PORTB, W
            btfss   STATUS, Z   ; preskoc pokud je tlacitko pustene             ; reverze
            goto    wait
            return              ; tlacitko je pustene, pokracuj

;-----------------------------------------------------------------------;
;                       Kontrola dvojkliku tl UNDEF                     ;
;-----------------------------------------------------------------------;
CheckDouble:
            call    releaseUNDEF
            call    msec250
;            movlw   DOUBLE_TIME
;CheckLoop:
;            nop
;            nop
;            nop
;            nop
;;            btfsc   SELECTB
;;            return
;            addlw   H'FF'       ; odcitani 1 od W
;            btfss   STATUS, Z   ; je W=0 (Z=1)?
;            goto    CheckLoop   ; ne, pokracuj v cekani
;CheckReturn:
            btfsc   UNDEFB
            call    SetTime
            call    releaseUNDEF
            return

;-----------------------------------------------------------------------;
;                              Inicializace                             ;
;-----------------------------------------------------------------------;
#include    Inicialization.inc

;-----------------------------------------------------------------------;
;     Obsluha preruseni (od TMR0), je potreba zvysit cas o vterinu      ;
;-----------------------------------------------------------------------;
#include    Interruption.inc

;-----------------------------------------------------------------------;
;           Zvysuj hodnotu jednotky casu na kterou odkazuje FSR         ;
;-----------------------------------------------------------------------;
UpDigit:
            incf    INDF, f     ; o jednicku cislo zvys
            movlw   week        ; Zjisti adresu registru
            subwf   FSR, W      ;  zpracovavane jednotky casu,
            call    SettingHi   ; jeho maximalni hodnotu +1 uloz do W.
            subwf   INDF, W     ; bylo teto
            btfsc   STATUS, Z   ;  hodnoty dosazeno?
            clrf    INDF        ; ano => vynuluj hodnotu

;;;;;;;;;;;;; zobrazeni indf a pote state
            movf    INDF, W     ; Novou hodnotu uloz do W
            call    SendZnak
            movlw   week
            subwf   FSR, W
            call    Katoda
            movwf   PORTA

            movlw   H'FF'
            call    nmsec       ; vyckej chvilu
            return
;; zobrazi nastavovanou hodnotu a reg state
ShowUpDigit:
            movf    INDF, W     ; Novou hodnotu uloz do W
            call    SendZnak
            movlw   week
            subwf   FSR, W
            call    Katoda
            movwf   PORTA
            movf    state, W
            call    SendZnak
            movlw   KAT_STATE
            movwf   PORTA

            movlw   D'8'
            call    nloop
            clrf    PORTA
            return

;-----------------------------------------------------------------------;
;         Zvysuj hodnotu jednotky casu dokud je tlacitko stisknute      ;
;-----------------------------------------------------------------------;
SetDigit:
            call    ShowUpDigit
            btfsc   SETB        ; je stisknute tl SET?                          ; reverze
            call    UpDigit     ; ano, zvysuj
            btfss   SELECTB     ; je stisknute tl SELECT?                       ; reverze
            goto    $ -4        ; opakuj dokud je tl SELECT stisknuto
            call    releaseSELECT     ; je opravdu pustene?
            decf    FSR, f
            return

;-----------------------------------------------------------------------;
;                          Obsluha behu stopek                          ;
;-----------------------------------------------------------------------;
Stopky:
            btfsc   bit_stavy, 0
            movf    state, W
            btfss   bit_stavy, 0
            movlw   D'0'
            call    SendZnak
            movlw   KAT_STATE
            movwf   PORTA

            btfss   STATEB
            goto    Stopky

            call    releaseSTATE
            call    SetState
;            movlw   R_1
;            movwf   state
            return

;-----------------------------------------------------------------------;
;                       Obsluha stisku tl STATE                         ;
;-----------------------------------------------------------------------;
SetState:
            incf    state, f
            call    releaseSTATE
            movlw   R_MAX
            subwf   state, W
            btfsc   STATUS, Z
            goto    $+2
            goto    $+3
            movlw   R_1
            goto    $+2
            movf    state, W
            movwf   state

            movf    state, W
            call    SendZnak
            movlw   KAT_STATE
            movwf   PORTA
            call    msec250

            return

;-----------------------------------------------------------------------;
;                   Obsluha rezimu zobrazeni                    ;
;-----------------------------------------------------------------------;
Rezim1:                         ; stale zobrazeni
            movlw   D'1'
            movwf   pom2
            clrf    por_reg
            call    DispTime
            clrf    PORTA
            return

Rezim2:                         ; blikajici zobrezeni
            movlw   D'19'       ; vytvori cas zobrazeni 1 vterinu (D'19'-nejpresnejsi k vterine)
            movwf   pom2
            clrf    por_reg
            clrf    PORTA
         ;   call    onesecond
            btfsc   bit_stavy, 0; zobrazuje jen kazdou lichou vterinu
            call    DispTime
            return

Rezim3:                         ; zobrezeni vypnuto
            clrf    W
            call    SendZnak
            return

SetTime:                        ; nastaveni data a casu
            bcf     INTCON, GIE ; zakazani preruseni
            clrf    por_reg
            movlw   hr          ; odkaz na registr hodin
            movwf   FSR

            movlw   H'08'
            movwf   state
            call    SetDigit

            movlw   H'04'
            movwf   state
            call    SetDigit

            movlw   H'02'
            movwf   state
            call    SetDigit

            movlw   H'01'
            movwf   state
            call    SetDigit


            movlw   R_1         ; presun na rezim staleho zobrazeni
            movwf   state
            bsf     INTCON, GIE ; znovu povol preruseni od TMR0
            return

Rezim4:                         ; stopky
;            btfss   SELECTB
            call    Stopky
            return
;-----------------------------------------------------------------------;
;                           Hlavni program                              ;
;-----------------------------------------------------------------------;
Main:
            call    Init        ; nastav pocatecni stav
MainLoop:
            btfsc   STATEB      ; pohotovostni stav, probiha vyber rezimu       ; reverze
            call    SetState

            btfss   UNDEFB
            goto    $+3
            movlw   R_1
            movwf   state

            movlw   R_1         ; rezim 1 - stale zobrazeni
            subwf   state, W
            btfsc   STATUS, Z
            call    Rezim1

            movlw   R_2         ; rezim 2 - blikajici zobrazeni
            subwf   state, W
            btfsc   STATUS, Z
            call    Rezim2      ; zobraz cas

            movlw   R_3         ; rezim 3 - zobrazeni vypnuto
            subwf   state, W
            btfsc   STATUS, Z
            call    Rezim3

            movlw   R_4         ; rezim 4 - stopky
            subwf   state, W
            btfsc   STATUS, Z
            call    Rezim4

;            movlw   R_5         ; rezim 5 - stopky
;            subwf   state, W
;            btfsc   STATUS, Z
;            call    Rezim5

            goto    MainLoop    ; opakuj hlavni smycku

            end